
unit Settings;

interface

procedure LoadSettings(
  out AUseBook: boolean;
  out
  APawnStructCo,
  ATargetsCo,
  ACastlingCo,
  AEnPassantCo,
  ACheckCo,
  ARevertCo: integer
);
procedure SaveSettings(
  const AUseBook: boolean;
  const
  APawnStructCo,
  ATargetsCo,
  ACastlingCo,
  AEnPassantCo,
  ACheckCo,
  ARevertCo: integer
);
function SettingsFileExists: boolean;

implementation

uses
  SysUtils, IniFiles;

const
  CUseBook = 1;
  CPawnStructCo = 1;
  CPieceTypeCo = 1;
  CCastlingCo = 10;
  CEnPassantCo = 10;
  CCheckCo = 1;
  CRevertCo = -100;
  CMain = 'main';
  CCoef = 'coefficients';
  
var
  LIniPath: string;

procedure LoadSettings(
  out AUseBook: boolean;
  out
  APawnStructCo,
  ATargetsCo,
  ACastlingCo,
  AEnPassantCo,
  ACheckCo,
  ARevertCo: integer
);
begin
  with TIniFile.Create(LIniPath) do
  try
    AUseBook := boolean(ReadInteger(CMain, 'usebook',      CUseBook));
    APawnStructCo    := ReadInteger(CCoef, 'pawnstruct',   CPawnStructCo);
    ATargetsCo       := ReadInteger(CCoef, 'targets',      CPieceTypeCo);
    ACastlingCo      := ReadInteger(CCoef, 'castling',     CCastlingCo);
    AEnPassantCo     := ReadInteger(CCoef, 'enpassant',    CEnPassantCo);
    ACheckCo         := ReadInteger(CCoef, 'check',        CCheckCo);
    ARevertCo        := ReadInteger(CCoef, 'revert',       CRevertCo);
  finally
    Free;
  end;
end;

procedure SaveSettings(
  const AUseBook: boolean;
  const
  APawnStructCo,
  ATargetsCo,
  ACastlingCo,
  AEnPassantCo,
  ACheckCo,
  ARevertCo: integer
);
begin
  with TIniFile.Create(LIniPath) do
  try
    WriteInteger(CMain, 'usebook',      Ord(AUseBook));
    WriteInteger(CCoef, 'pawnstruct',   APawnStructCo);
    WriteInteger(CCoef, 'targets',      ATargetsCo);
    WriteInteger(CCoef, 'castling',     ACastlingCo);
    WriteInteger(CCoef, 'enpassant',    AEnPassantCo);
    WriteInteger(CCoef, 'check',        ACheckCo);
    WriteInteger(CCoef, 'revert',       ARevertCo);
    UpdateFile;
  finally
    Free;
  end;
end;

function SettingsFileExists: boolean;
begin
  result := FileExists(LIniPath);
end;

begin
  LIniPath := ChangeFileExt(ParamStr(0), '.cfg');
end.
